"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSubscription = exports.isClass = exports.fetchIntrospectionSchema = exports.stripObject = exports.getUrl = exports.getSchemaQuery = exports.getSchemaMutation = exports.getSchemaTypeFields = exports.getParsedSchema = exports.config = exports.registryGet = exports.performSafeRequestGraphql = exports.stripTypename = exports.performGqlSubscription = exports.performGqlMutation = exports.performGqlQuery = exports.performSafeRequestREST = exports.json = exports.status = exports.queryParams = exports.camelToKebab = exports.getApolloClient = exports.defaultRESTHeaders = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("graphql");
const parser_1 = require("graphql/language/parser");
const graphql_2 = require("graphql");
const Registry_1 = tslib_1.__importDefault(require("./Registry"));
const UnexpectedException_1 = tslib_1.__importDefault(require("./models/Exceptions/UnexpectedException"));
const apolloClient_1 = require("./graphql/apolloClient");
exports.defaultRESTHeaders = {
    'Accept': 'application/json',
    'Content-Type': 'application/json;charset=UTF-8',
};
function getApolloClient() {
    return apolloClient_1.apolloClient;
}
exports.getApolloClient = getApolloClient;
function camelToKebab(input) {
    return input.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
}
exports.camelToKebab = camelToKebab;
function queryParams(params, camelToKebabActive = true) {
    const str = [];
    for (const paramsKey in params)
        if (params.hasOwnProperty(paramsKey) && (params[paramsKey] || params[paramsKey] === 0)) {
            const normalized = camelToKebabActive ? camelToKebab(encodeURIComponent(paramsKey)) : encodeURIComponent(paramsKey);
            str.push(`${normalized}=${encodeURIComponent(params[paramsKey])}`);
        }
    return str.join('&');
}
exports.queryParams = queryParams;
function status(res) {
    const response = res.hasOwnProperty('status') ? res : res.response;
    if (response.status && response.status >= 200 && response.status < 300) {
        return Promise.resolve(response);
    }
    else {
        if (response.data && response.data.errors) {
            const errors = (response.data.errors || []).map((err) => err.defaultMessage).join(', ');
            return Promise.reject(new Error(errors));
        }
        return Promise.reject(new Error(response.data.message));
    }
}
exports.status = status;
function json(response) {
    if (response && response.data) {
        return response.data;
    }
    else {
        return null;
    }
}
exports.json = json;
async function performSafeRequestREST(url, params = {}, method = 'get', opts = {}) {
    let fullUrl = url;
    let body = {};
    if (method === 'get') {
        const queryEscaped = queryParams(params);
        if (queryEscaped) {
            fullUrl = `${fullUrl}?${queryEscaped}`;
        }
    }
    else {
        body = params;
    }
    return fetch(fullUrl, {
        method: method.toLowerCase(),
        headers: exports.defaultRESTHeaders,
        body: JSON.stringify(body),
        ...opts,
    }).then(status).then(json);
}
exports.performSafeRequestREST = performSafeRequestREST;
function performGqlQuery(query, variables) {
    return getApolloClient().query({
        query,
        variables,
    });
}
exports.performGqlQuery = performGqlQuery;
function performGqlMutation(mutation, variables) {
    return getApolloClient().mutate({
        mutation,
        variables,
    });
}
exports.performGqlMutation = performGqlMutation;
async function performGqlSubscription(subscription, variables) {
    return getApolloClient().subscribe({
        query: subscription,
        variables,
    });
}
exports.performGqlSubscription = performGqlSubscription;
function stripTypename(obj) {
    return JSON.parse(JSON.stringify(obj, (k, v) => (k === '__typename' ? undefined : v)));
}
exports.stripTypename = stripTypename;
async function performSafeRequestGraphql(query, variables = {}) {
    const queryName = query.definitions.map(def => def.name).find(def => def.kind === 'Name').value;
    const isQuery = query.definitions.find(def => def.kind === 'OperationDefinition').operation === 'query';
    if (isQuery) {
        return performGqlQuery(query, stripTypename(variables))
            .then((value) => value.data[queryName]);
    }
    const isSubscription = query.definitions.find(def => def.kind === 'OperationDefinition').operation === 'subscription';
    if (isSubscription) {
        return performGqlSubscription(query, stripTypename(variables));
    }
    return performGqlMutation(query, stripTypename(variables))
        .then((value) => value.data[queryName]);
}
exports.performSafeRequestGraphql = performSafeRequestGraphql;
function registryGet(key) {
    return Registry_1.default.getInstance().get(key);
}
exports.registryGet = registryGet;
function config() {
    return registryGet('Config');
}
exports.config = config;
async function getParsedSchema() {
    const configSchema = config().schema;
    const configSchemaUrl = config().schemaUrl;
    let schema = Registry_1.default.getInstance().get('schema');
    if (!schema && configSchema) {
        schema = configSchema;
    }
    else if (!schema && configSchemaUrl) {
        schema = await fetchIntrospectionSchema(configSchemaUrl)
            .then(graphql_1.buildClientSchema.bind(null))
            .then(graphql_1.printSchema.bind(null))
            .then(parser_1.parse.bind(null));
        Registry_1.default.getInstance().set('schema', schema);
    }
    if (!schema) {
        throw new UnexpectedException_1.default('Configuration error: \'schema\' must be passed as a config key, e.g\n\nimport schema from \'raw-loader!@/../schema.graphql\';\n\n//...\n\nVue.use(VueOOP, {\n  //...,\n  schema,\n})\n\n;');
    }
    return schema;
}
exports.getParsedSchema = getParsedSchema;
async function getSchemaTypeFields(typeName) {
    return (await getParsedSchema())
        .definitions
        .find(def => (def.name || {}).value === typeName)
        .fields
        .map(f => f.name.value);
}
exports.getSchemaTypeFields = getSchemaTypeFields;
async function getSchemaMutation(mutationName) {
    return (await getParsedSchema())
        .definitions
        .find(def => (def.name || {}).value === 'Mutation')
        .fields
        .find(def => (def.name || {}).value === mutationName);
}
exports.getSchemaMutation = getSchemaMutation;
async function getSchemaQuery(queryName) {
    return (await getParsedSchema())
        .definitions
        .find(def => (def.name || {}).value === 'Query')
        .fields
        .find(def => (def.name || {}).value === queryName);
}
exports.getSchemaQuery = getSchemaQuery;
async function getUrl(_opts) {
    const { url, params } = _opts;
    let resolvedUrl = url;
    if (typeof url === 'function') {
        resolvedUrl = await url();
    }
    else {
        resolvedUrl = resolvedUrl.replace(/:([^\s\/?&]+)/gi, (_, m) => {
            const param = params[m];
            const hasParam = param !== undefined;
            if (hasParam) {
                delete params[m];
                return param;
            }
            return m;
        });
    }
    return resolvedUrl;
}
exports.getUrl = getUrl;
function stripObject(obj) {
    return JSON.parse(JSON.stringify(obj, (k, v) => (k === 'loading' ? undefined : v)));
}
exports.stripObject = stripObject;
function fetchIntrospectionSchema(url) {
    const body = JSON.stringify({ query: graphql_2.getIntrospectionQuery() });
    return fetch(url, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body
    })
        .then(res => res.json())
        .then(res => res.data);
}
exports.fetchIntrospectionSchema = fetchIntrospectionSchema;
exports.isClass = (fn) => /^\s*class/.test(fn.toString());
exports.isSubscription = (data) => data._subscriber;
//# sourceMappingURL=utils.js.map